[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet Estimates Report Builder client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.0-blue)](https://developer.factset.com/api-catalog/factset-estimates-report-builder-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetEstimatesReportBuilder/1.0.14)](https://pypi.org/project/fds.sdk.FactSetEstimatesReportBuilder/v/1.0.14)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

License info

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.0
- SDK version: 1.0.14
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetEstimatesReportBuilder==1.0.14
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetEstimatesReportBuilder==1.0.14
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetEstimatesReportBuilder
from fds.sdk.FactSetEstimatesReportBuilder.api import all_estimates_api
from fds.sdk.FactSetEstimatesReportBuilder.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetEstimatesReportBuilder.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetEstimatesReportBuilder.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetEstimatesReportBuilder.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = all_estimates_api.AllEstimatesApi(api_client)
    id = "FDS" # str | Company Ticker
    periodicity = "ANN" # str | Periodicity or frequency of the fiscal periods. (optional) if omitted the server will use the default value of "ANN"
    schema = "table_parent_child_columns" # str | The schema that the data is returned as. The following are descriptions for the accepted values: - table_group_level - STACH 2.0 row organized package format with parent-child relationships represented using STACH group level cell metadata - table_parent_child_columns - STACH 2.0 row organized package format with parent-child relationships represented using STACH parent-child columns  (optional) if omitted the server will use the default value of "table_parent_child_columns"

    try:
        # Balance Sheet
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_estimates_balance_sheet(id, periodicity=periodicity, schema=schema)

        pprint(api_response)
    except fds.sdk.FactSetEstimatesReportBuilder.ApiException as e:
        print("Exception when calling AllEstimatesApi->get_estimates_balance_sheet: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Balance Sheet
    #     api_response, http_status_code, response_headers = api_instance.get_estimates_balance_sheet_with_http_info(id, periodicity=periodicity, schema=schema)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetEstimatesReportBuilder.ApiException as e:
    #     print("Exception when calling AllEstimatesApi->get_estimates_balance_sheet: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Balance Sheet
    #     async_result = api_instance.get_estimates_balance_sheet_async(id, periodicity=periodicity, schema=schema)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.FactSetEstimatesReportBuilder.ApiException as e:
    #     print("Exception when calling AllEstimatesApi->get_estimates_balance_sheet: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Balance Sheet
    #     async_result = api_instance.get_estimates_balance_sheet_with_http_info_async(id, periodicity=periodicity, schema=schema)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetEstimatesReportBuilder.ApiException as e:
    #     print("Exception when calling AllEstimatesApi->get_estimates_balance_sheet: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetEstimatesReportBuilder

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetEstimatesReportBuilder.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetEstimatesReportBuilder

configuration = fds.sdk.FactSetEstimatesReportBuilder.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetEstimatesReportBuilder

configuration = fds.sdk.FactSetEstimatesReportBuilder.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetEstimatesReportBuilder

configuration = fds.sdk.FactSetEstimatesReportBuilder.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/report/estimates/v1*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*AllEstimatesApi* | [**get_estimates_balance_sheet**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/AllEstimatesApi.md#get_estimates_balance_sheet) | **GET** /balance-sheet | Balance Sheet
*AllEstimatesApi* | [**get_estimates_cash_flow**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/AllEstimatesApi.md#get_estimates_cash_flow) | **GET** /cash-flow | Cash Flow
*AllEstimatesApi* | [**get_estimates_geographic_segments**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/AllEstimatesApi.md#get_estimates_geographic_segments) | **GET** /geographic-segments | Geographic Segments
*AllEstimatesApi* | [**get_estimates_income_statement**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/AllEstimatesApi.md#get_estimates_income_statement) | **GET** /income-statement | Income Statement
*AllEstimatesApi* | [**get_estimates_industry_metrics**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/AllEstimatesApi.md#get_estimates_industry_metrics) | **GET** /industry-metrics | Industry Metrics
*AllEstimatesApi* | [**get_estimates_per_share**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/AllEstimatesApi.md#get_estimates_per_share) | **GET** /per-share | Per Share
*AllEstimatesApi* | [**get_estimates_product_segments**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/AllEstimatesApi.md#get_estimates_product_segments) | **GET** /product-segments | Product Segments
*AllEstimatesApi* | [**get_estimates_valuation**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/AllEstimatesApi.md#get_estimates_valuation) | **GET** /valuation | Valuation
*EstimateTablesApi* | [**get_estimate_tables_table**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/EstimateTablesApi.md#get_estimate_tables_table) | **GET** /table | Interim/Annual Estimate Table


## Documentation For Models

 - [ErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/ErrorObject.md)
 - [ErrorObjectLinks](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/ErrorObjectLinks.md)
 - [ErrorObjectSource](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/ErrorObjectSource.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/ErrorResponse.md)
 - [Meta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/Meta.md)
 - [Response](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimatesReportBuilder/v1/docs/Response.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetEstimatesReportBuilder.apis and fds.sdk.FactSetEstimatesReportBuilder.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetEstimatesReportBuilder.api.default_api import DefaultApi`
- `from fds.sdk.FactSetEstimatesReportBuilder.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetEstimatesReportBuilder
from fds.sdk.FactSetEstimatesReportBuilder.apis import *
from fds.sdk.FactSetEstimatesReportBuilder.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

